#import "OskarView.h"
#import "ServiceUSBToolbox.h"

@implementation OskarView

- (IBAction)speedSliderChanged:(id)sender
{
  // for a better feeling of speed slider use it in a non-linear way
  int value;
  value = [sender floatValue];
  value = (value * value)/2000;
  if (oskarParams.displaySpeed != value){
    oskarParams.displaySpeed = value;
    [textFeld setIntValue:oskarParams.displaySpeed];
    [self prepareRecordData];
  }    
}

- (IBAction)startStopButtonChanged:(id)sender
{
 if ((oskarParams.displayEX = [[sender cellWithTag:0]state]) +
      (oskarParams.displayEY = [[sender cellWithTag:1]state]) +
      (oskarParams.displayDigital = [[sender cellWithTag:2]state]))	//start record
    oskarParams.recordStop = NO;    
  else																															// stop record
    oskarParams.recordStop = YES;
        
  [self prepareRecordData];
}

- (IBAction)triggerButtonChanged:(id)sender
{
  // trigger not yet implemented
  if (oskarParams.triggerChannel != [[sender selectedCell]tag]){
    oskarParams.triggerChannel = [[sender selectedCell]tag];
    [self prepareRecordData];
  }
}

- (IBAction)triggerSliderChanged:(id)sender
{
  // trigger not yet implemented
  if ([sender intValue] != oskarParams.triggerLevel){
    oskarParams.triggerLevel = [sender intValue];
    [self prepareRecordData];
  }
}

- (id)initWithFrame:(NSRect)frameRect
{
  self = [super initWithFrame:frameRect];
  if (self){
    exPath = [[NSBezierPath alloc] init];
    eyPath = [[NSBezierPath alloc] init];
    digitalPath = [[NSBezierPath alloc] init];

    oskarParams.displayEX = NO;
    oskarParams.displayEY = NO;
    oskarParams.displayDigital = NO;
    oskarParams.recordStop = YES;
    oskarParams.displaySlowSpeed = NO;
    oskarParams.triggerChannel = 0;
    oskarParams.triggerLevel = 0;
    oskarParams.displaySpeed = (1000 * 1000) / 2000;
    oskarParams.xAchse = 0;       
  }
  return self;
}

- (void)dealloc
{
  [digitalPath release];
  [eyPath release];
  [exPath release];
  [super release];
}

-(void)drawRect:(NSRect)aRect
{
  [[NSColor whiteColor] set];				// first clear drawing area to white
  [NSBezierPath fillRect:aRect];
  
  if (oskarParams.displayEX){				// draw EX if display switch is on
    [[exColor color]set];
    [exPath stroke];
  }
  if (oskarParams.displayEY){				// draw EY if display switch is on
    [[eyColor color]set];
    [eyPath stroke];
  }
  if (oskarParams.displayDigital){	// draw digital if display switch is on
    [[digitalColor color]set];
    [digitalPath stroke];
  }

}
- (void) startTimer
{
  if (!recordTimer ){				// first Time to start Timer
    recordTimer = [[NSTimer	scheduledTimerWithTimeInterval:1.0/oskarParams.displaySpeed
                            target:self
                            selector:@selector(recordData:)
                            userInfo:nil
                            repeats:YES]retain];
  }
  else {									// Timer already running. First stop, then start again
    [recordTimer invalidate];
    [recordTimer release];
    recordTimer = [[NSTimer	scheduledTimerWithTimeInterval:1.0/oskarParams.displaySpeed
                            target:self
                            selector:@selector(recordData:)
                            userInfo:nil
                            repeats:YES] retain];
  }
}
- (void) stopTimer
{
  if (recordTimer){
    [recordTimer invalidate];
    [recordTimer release];
    recordTimer = nil;
  }
}
- (void) prepareRecordData
{
  oskarParams.xAchse = 0;
  
  if (oskarParams.recordStop == NO){
    if (oskarParams.displaySpeed > 80)		// more than 80 Values/second
      oskarParams.displaySlowSpeed = NO;	// first record all values, then display
    else
      oskarParams.displaySlowSpeed = YES;	// less than 80 Values/s
                                          // display every value on the screen immediately
    [self startTimer];
  }
  else
    [self stopTimer];
}
- (void) recordData:(id)sender						// Timer calls this to record all data
{
  static NSTimeInterval timeBevore = 0;
  NSPoint movePoint;
  
  if (oskarParams.xAchse == 0)
    timeBevore = [NSDate timeIntervalSinceReferenceDate];

  movePoint.x = oskarParams.xAchse;

  if (movePoint.x == 0){
    movePoint.y = ServiceUSB_GetEXValue((ServiceUSB_DeviceIdentifierRef)0);
    [exPath removeAllPoints];
    [exPath moveToPoint:movePoint];
    movePoint.y = ServiceUSB_GetEYValue((ServiceUSB_DeviceIdentifierRef)0);
    [eyPath removeAllPoints];
    [eyPath moveToPoint:movePoint];
    movePoint.y = ServiceUSB_GetInputValue((ServiceUSB_DeviceIdentifierRef)0);
    [digitalPath removeAllPoints];
    [digitalPath moveToPoint:movePoint];
  }
  else{
    movePoint.y = ServiceUSB_GetEXValue((ServiceUSB_DeviceIdentifierRef)0);
    [exPath lineToPoint:movePoint];
    movePoint.y = ServiceUSB_GetEYValue((ServiceUSB_DeviceIdentifierRef)0);
    [eyPath lineToPoint:movePoint];
    movePoint.y = ServiceUSB_GetInputValue((ServiceUSB_DeviceIdentifierRef)0);
    [digitalPath lineToPoint:movePoint];
  }
  
  if (oskarParams.displaySlowSpeed == YES)
    [self setNeedsDisplay:YES];
  if (oskarParams.xAchse++ > [self bounds].size.width){
    if (oskarParams.displaySlowSpeed == NO)
      [self setNeedsDisplay:YES];
    timeNeeded = [NSDate timeIntervalSinceReferenceDate] - timeBevore;
    [textFeld setIntValue:[self bounds].size.width / timeNeeded];
    oskarParams.xAchse = 0;
  }
  
}

@end